==> lowcode-render-1.1.3.tgz: package/src/app/global-api.ts <==
import {
  createWebApp,
  IWebAppAPI,
  _ACTIONS_KEY as ACTIONS_KEY,
  _ROUTER_KEY as ROUTER_KEY,
  actionSdk as sdk,
  // _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK,
} from '@cloudbase/weda-client';
import { runtime } from '../utils/adapter/index';
import { get as lodashGet } from 'lodash-es';
import { OFFICIAL_LIB_KEY } from '@cloudbase/cals';
import { observable } from 'mobx';
import { UserWidget, bindFuncObj, createComputed } from '../utils';
import * as querystring from 'query-string';

interface IBaseState {
  state: {
    [key: string]: any;
  };
  computed: {
    [key: string]: any;
  };
  dataset?: {
    state?: {};
    params?: {};
    query?: {};
  };
}

// enum EPLATFORM {
//   WEB = 'WEB',
//   /**
//    * 拼写错误
//    * 兼容性无法修改
//    */
//   MINIPROGRAM = 'MINIPROGRAME',
// }

const DEFAULT_INIT_DATA = {
  processAction: (key, action) => {
    switch (key) {
      case ACTIONS_KEY.showModal: {
        return function (obj) {
          const showModal = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showModal;
          const isMobile = app?.__internal__?.isMobile;
          if (isMobile && !isMobile() && showModal) {
            return showModal({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
      case ACTIONS_KEY.showToast: {
        return function (obj) {
          const showMessage = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showMessage;
          const isMobile = app?.__internal__?.isMobile;
          // pc端toast和loading为不同实例，所以展示toast时先将loading隐藏掉
          if (isMobile && !isMobile()) {
            app?.hideLoading?.();
          }
          if (showMessage) {
            return showMessage({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
      case ACTIONS_KEY.showLoading: {
        return function (obj) {
          const isMobile = app?.__internal__?.isMobile;
          if (isMobile && !isMobile()) {
            // pc端toast和loading为不同实例，所以展示loading时先将toast隐藏掉
            try {
              const message = document.getElementById('tea-message-root');
              (message.children[0] as any).style.display = 'none';
            } catch (error) {
              //
            }
            return action(obj, 'pc');
          }
          return action(obj);
        };
      }
    }
    return action;
  },
};

export const subPackageName = '';
export let app = createGlboalApi(DEFAULT_INIT_DATA);
export const $w = new Proxy(app.__internal__.$w, {
  get(_, prop: string) {
    if (prop === '$app' || prop === 'app') {
      return app;
    }
    return app.__internal__.$w?.[prop];
  },
});

export function registerApp(_app = createGlboalApi(DEFAULT_INIT_DATA)) {
  _injectApp2Runtime(_app, true);
  app = _app;
}

mountAPIs(sdk);
export interface IAppAPI extends IWebAppAPI, IBaseState {
  domain: string;
  app?: IAppAPI;
  $app?: IAppAPI;
  mainApp?: IAppAPI;
  common: {};
  /**
   * 内置方法
   */
  setState: (state) => void;
}

function createGlboalApi(
  initData: {
    id?: string;
    domain?: string;
    appConfig?: {
      id?: string;
      envId?: string;
      envVersion?: 'preview' | 'production';
      staticResourceDomain?: string;
      pages: { id: string }[];
    };
    processAction?: (key, action) => any;
  } = {},
): IAppAPI {
  const webApp = createWebApp(initData);
  const globalAPI: IAppAPI = new Proxy(
    Object.assign(webApp, {
      domain: initData.domain || '',
      state: {},
      computed: {},
      dataset: {},
      common: new Proxy(
        {},
        {
          get(_, prop) {
            console.warn('$app.common 还未初始化完毕。应避免在低码函数外部直接对 $app.common.xxx 发起访问。');
            return _[prop];
          },
        },
      ),
      // i18n,
      ...webApp,
      utils: {
        ...webApp.utils,
      },
      /**
       * setState
       */
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          globalAPI.utils.set(globalAPI.dataset.state, keyPath, userSetState[keyPath]);
        });
      },
    }),
    {
      get: (obj, prop) => {
        if (prop === 'app' || prop === '$app') {
          return app;
        } else {
          return obj[prop];
        }
      },
      // set: (obj, prop, value) => {
      //   if (prop === 'app' || prop === '$app') {
      //     console.warn('$app 对象中 app/$app 为只读');
      //     return false;
      //   } else {
      //     // 修改 obj
      //     return true;
      //   }
      // },
    },
  );
  _injectApp2Runtime(globalAPI);

  return globalAPI;
}

function _injectApp2Runtime(globalAPI, hard = false) {
  // 给全局挂上 mainApp/subApp
  // The global api exposed to lowcode
  if (subPackageName) {
    globalAPI.mainApp = runtime.$app;
    runtime[`$$subapp_${subPackageName}`] = globalAPI;
  } else {
    // 预览区调试栏可调试
    // 暴露给复合组件，代码组件来引用
    runtime.app = hard ? globalAPI : runtime.app || globalAPI;
    runtime.$app = new Proxy(runtime.app, {
      get(obj, prop) {
        return obj ? obj[prop] : undefined;
      },
      set(obj, prop, value) {
        if (obj) {
          return (obj[prop] = value);
        } else {
          return undefined;
        }
      },
    });

    runtime.$w = new Proxy($w || globalAPI?.__internal__?.$w || {}, {
      get(_, prop: string) {
        const $page = runtime.app?.__internal__?.activePage || runtime.$page;
        if ($page?.__internal__?.$w?.[prop]) {
          return $page?.__internal__?.$w?.[prop];
        }

        if (_[prop]) {
          return _[prop];
        }

        return null;
      },
    });
  }
}

export function removeGlboalApi() {
  if (subPackageName) {
    runtime[`$$subapp_${subPackageName}`] = undefined;
  } else {
    runtime.app = undefined;
    runtime.$app = undefined;
  }
}

export interface IPageAPI extends IBaseState {
  __internal__?: {
    $w?: any;
    active: boolean;
    packageName?: string;
  };
  $page?: IPageAPI;
  id?: string;
  uuid?: string;
  handler: {
    [key: string]: any;
  };
  widgets: {
    [key: string]: any;
  };
  setState: (state) => void;
  setParams: (params) => void;
  /**
   * @deprecated
   */
  invokeComponentMethod: (props: { component: string | UserWidget; method: string; params: any }) => any;
  _dataBinds: {
    [key: string]: any;
  };
}

export function createPageApi(): IPageAPI {
  const $page: IPageAPI = new Proxy(
    {
      __internal__: {
        active: false,
      },
      state: {},
      computed: {},
      handler: {},
      widgets: observable({}),
      dataset: {},
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          app.utils.set($page.dataset?.state, keyPath, userSetState[keyPath]);
        });
      },
      setParams: (_params) => {
        const params = Array.isArray(_params)
          ? _params.reduce((map, meta) => {
              let { key, value } = meta;
              if (key?.startsWith?.('$page.')) {
                key = key.replace(/^\$page\./, '');
              }
              if (key?.trim?.()) {
                map[key?.trim?.()] = value;
              }
              return map;
            }, {})
          : _params;

        Object.keys(params).forEach((keyPath) => {
          app.utils.set($page.dataset?.params, keyPath, params[keyPath]);
        });

        let queryObj;
        try {
          queryObj =
            querystring.parse(((runtime as any)._WEAPPS_HISTORY.location.search || '').split('?')[1] || '') || {};
        } catch (e) {
          queryObj = { ...$page.dataset?.params };
        }

        Object.keys(params)
          .map((keyPath) => {
            const matched = keyPath.match(/(^.*?)[.[]/);
            if (matched) {
              return matched[0];
            }
            return keyPath;
          })
          .forEach((key) => {
            queryObj[key] = $page.dataset?.params?.[key];
          });

        app.redirectTo({
          pageId: $page.id,
          packageName: $page.__internal__.packageName,
          params: queryObj,
        });
      },
      _dataBinds: {},
      invokeComponentMethod({ component, method, params }) {
        let userWidget: UserWidget;
        if (typeof component === 'string') {
          const widgetData = lodashGet($page.widgets, component);
          if (Array.isArray(widgetData)) {
            throw new Error(`调用方法失败：id为${component}的组件拥有多个实例`);
          }
          userWidget = widgetData?._userWidget;
        } else {
          userWidget = component;
        }
        const action = userWidget?.custom?.[method];
        if (typeof action !== 'function') {
          throw new Error(`调用方法失败：未找到id为${userWidget?.sys.id}下的方法${method}`);
        }
        return action(params);
      },
    },
    {
      get(obj, prop) {
        return prop === '$page' ? obj : obj[prop];
      },
    },
  );
  return $page;
}

// 分app 和 wx 挂载app
function mountAPIs(sdks) {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];
    if (!(item in ACTIONS_KEY) && !(item in ROUTER_KEY)) {
      app[item] = action;
    }
  });
  return app;
}

export interface ICompositeComponentAPI extends IBaseState {
  __internal__?: {
    $w?: any;
    defaultCssUnit?: 'rpx' | 'rem' | 'px';
  };
  id: string;
  handler?: {
    [key: string]: any;
  };
  props: {
    [key: string]: any;
  };
  widgets?: {
    [key: string]: any;
  };
  _dataBinds?: {
    [key: string]: any;
  };
  _instanceRef: { current: any };
  compConfig?: Function;
}

export function createCompApi({
  id,
  ref,
  state = observable({}),
  computed = {},
  _dataBinds = {},
  compConfig = {},
  node,
  events,
  props,
  handler = {},
  propsData,
}): ICompositeComponentAPI {
  const $comp = new Proxy(
    {
      id,
      __internal__: {
        defaultCssUnit: undefined,
        $w: new Proxy(
          {},
          {
            get(_, prop) {
              /**
               * 代理可以访问到当前 page
               */
              if (prop === '$page' || prop === 'page') {
                return app?.__internal__.activePage;
              }

              // 尝试代理页面级别组件实例
              let pageWidget = $comp.widgets?.[prop];
              if (pageWidget) {
                return pageWidget._userWidget;
              }

              return app?.__internal__?.$w?.[prop];
            },
          },
        ),
      },
      _dataBinds,
      compConfig,
      widgets: observable({}),
      node,
      // lib: { const: constObj, tools: toolsObj },
      get props() {
        const { forwardRef, ...restProps } = props || {};
        return { ...restProps, events, data: propsData };
      },
      get state() {
        return state;
      },
      computed: {},
      get _instanceRef() {
        return ref;
      },
      get methods() {
        return ref.current?.methods;
      },
      set methods(value) {
        if (!ref.current) {
          ref.current = {};
        }
        ref.current.methods = value;
      },
    },
    {
      get: (obj, prop) => {
        return prop === '$WEAPPS_COMP' || prop === '$comp' ? obj : obj[prop];
      },
    },
  );

  return Object.assign($comp, {
    computed: createComputed(computed, $comp),
    handler: bindFuncObj(handler || {}, $comp),
  });
}

==> lowcode-render-1.1.4.tgz: package/src/app/global-api.ts <==
import {
  createWebApp,
  IWebAppAPI,
  _ACTIONS_KEY as ACTIONS_KEY,
  _ROUTER_KEY as ROUTER_KEY,
  actionSdk as sdk,
  getWedaAPI,
  // _WEDA_CLOUD_SDK as WEDA_CLOUD_SDK,
} from '@cloudbase/weda-client';
import { runtime } from '../utils/adapter/index';
import { get as lodashGet } from 'lodash-es';
import { OFFICIAL_LIB_KEY } from '@cloudbase/cals';
import { observable } from 'mobx';
import { UserWidget, bindFuncObj, createComputed } from '../utils';
import * as querystring from 'query-string';
import { generateEventFlows } from '../utils/adapter/generate';

interface IBaseState {
  state: {
    [key: string]: any;
  };
  computed: {
    [key: string]: any;
  };
  dataset?: {
    state?: {};
    params?: {};
    query?: {};
  };
}

// enum EPLATFORM {
//   WEB = 'WEB',
//   /**
//    * 拼写错误
//    * 兼容性无法修改
//    */
//   MINIPROGRAM = 'MINIPROGRAME',
// }

const DEFAULT_INIT_DATA = {
  processAction: (key, action) => {
    switch (key) {
      case ACTIONS_KEY.showModal: {
        return function (obj) {
          const showModal = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showModal;
          const isMobile = app?.__internal__?.isMobile;
          if (isMobile && !isMobile() && showModal) {
            return showModal({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
      case ACTIONS_KEY.showToast: {
        return function (obj) {
          const showMessage = runtime[`@weapps-materials-main-${OFFICIAL_LIB_KEY}`]?.actions?.showMessage;
          const isMobile = app?.__internal__?.isMobile;
          // pc端toast和loading为不同实例，所以展示toast时先将loading隐藏掉
          if (isMobile && !isMobile()) {
            app?.hideLoading?.();
          }
          if (showMessage) {
            return showMessage({ data: obj, _origin: action });
          }
          return action(obj);
        };
      }
      case ACTIONS_KEY.showLoading: {
        return function (obj) {
          const isMobile = app?.__internal__?.isMobile;
          if (isMobile && !isMobile()) {
            // pc端toast和loading为不同实例，所以展示loading时先将toast隐藏掉
            try {
              const message = document.getElementById('tea-message-root');
              (message.children[0] as any).style.display = 'none';
            } catch (error) {
              //
            }
            return action(obj, 'pc');
          }
          return action(obj);
        };
      }
    }
    return action;
  },
};

export const subPackageName = '';
let __app = undefined;
export const $w = new Proxy(__app?.__internal__?.$w || {}, {
  get(_, prop: string) {
    if (prop === '$app' || prop === 'app') {
      return __app;
    }
    return __app?.__internal__?.$w?.[prop];
  },
});
__app = createGlboalApi(DEFAULT_INIT_DATA);
export let app = __app;

export function registerApp(_app = createGlboalApi(DEFAULT_INIT_DATA)) {
  _injectApp2Runtime(_app, true);
  app = _app;
}

mountAPIs(sdk);
export interface IAppAPI extends IWebAppAPI, IBaseState {
  domain: string;
  app?: IAppAPI;
  $app?: IAppAPI;
  mainApp?: IAppAPI;
  common: {};
  /**
   * 内置方法
   */
  setState: (state) => void;
}

function createGlboalApi(
  initData: {
    id?: string;
    domain?: string;
    appConfig?: {
      id?: string;
      envId?: string;
      envVersion?: 'preview' | 'production';
      staticResourceDomain?: string;
      pages: { id: string }[];
    };
    processAction?: (key, action) => any;
  } = {},
): IAppAPI {
  const webApp = createWebApp(initData);
  const globalAPI: IAppAPI = new Proxy(
    Object.assign(webApp, {
      domain: initData.domain || '',
      state: {},
      computed: {},
      dataset: {},
      common: new Proxy(
        {},
        {
          get(_, prop) {
            console.warn('$app.common 还未初始化完毕。应避免在低码函数外部直接对 $app.common.xxx 发起访问。');
            return _[prop];
          },
        },
      ),
      // i18n,
      ...webApp,
      utils: {
        ...webApp.utils,
      },
      /**
       * setState
       */
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          globalAPI.utils.set(globalAPI.dataset.state, keyPath, userSetState[keyPath]);
        });
      },
    }),
    {
      get: (obj, prop) => {
        if (prop === 'app' || prop === '$app') {
          return app;
        } else {
          return obj[prop];
        }
      },
      // set: (obj, prop, value) => {
      //   if (prop === 'app' || prop === '$app') {
      //     console.warn('$app 对象中 app/$app 为只读');
      //     return false;
      //   } else {
      //     // 修改 obj
      //     return true;
      //   }
      // },
    },
  );
  _injectApp2Runtime(globalAPI);

  return globalAPI;
}

function _injectApp2Runtime(globalAPI, hard = false) {
  // 给全局挂上 mainApp/subApp
  // The global api exposed to lowcode
  if (subPackageName) {
    globalAPI.mainApp = runtime.$app;
    runtime[`$$subapp_${subPackageName}`] = globalAPI;
  } else {
    // 预览区调试栏可调试
    // 暴露给复合组件，代码组件来引用
    runtime.app = hard ? globalAPI : runtime.app || globalAPI;
    runtime.$app = new Proxy(runtime.app, {
      get(obj, prop) {
        return obj ? obj[prop] : undefined;
      },
      set(obj, prop, value) {
        if (obj) {
          return (obj[prop] = value);
        } else {
          return undefined;
        }
      },
    });

    runtime.$w = new Proxy($w || globalAPI?.__internal__?.$w || {}, {
      get(_, prop: string) {
        const $page = runtime.app?.__internal__?.activePage || runtime.$page;

        if ($page?.__internal__?.$w?.[prop]) {
          return $page.__internal__.$w[prop];
        }

        if (_[prop]) {
          return _[prop];
        }

        return null;
      },
    });
  }
}

export function removeGlboalApi() {
  if (subPackageName) {
    runtime[`$$subapp_${subPackageName}`] = undefined;
  } else {
    runtime.app = undefined;
    runtime.$app = undefined;
  }
}

export interface IPageAPI extends IBaseState {
  __internal__?: {
    $w?: any;
    active: boolean;
    packageName?: string;
  };
  $page?: IPageAPI;
  id?: string;
  uuid?: string;
  handler: {
    [key: string]: any;
  };
  widgets: {
    [key: string]: any;
  };
  setState: (state) => void;
  setParams: (params) => void;
  /**
   * @deprecated
   */
  invokeComponentMethod: (props: { component: string | UserWidget; method: string; params: any }) => any;
  _dataBinds: {
    [key: string]: any;
  };
}

export function createPageApi(): IPageAPI {
  const $page: IPageAPI = new Proxy(
    {
      __internal__: {
        active: false,
      },
      state: {},
      computed: {},
      handler: {},
      widgets: observable({}),
      dataset: {},
      setState: (userSetState) => {
        Object.keys(userSetState).forEach((keyPath) => {
          app.utils.set($page.dataset?.state, keyPath, userSetState[keyPath]);
        });
      },
      setParams: (_params) => {
        const params = Array.isArray(_params)
          ? _params.reduce((map, meta) => {
              let { key, value } = meta;
              if (key?.startsWith?.('$page.')) {
                key = key.replace(/^\$page\./, '');
              }
              if (key?.trim?.()) {
                map[key?.trim?.()] = value;
              }
              return map;
            }, {})
          : _params;

        Object.keys(params).forEach((keyPath) => {
          app.utils.set($page.dataset?.params, keyPath, params[keyPath]);
        });

        let queryObj;
        try {
          queryObj =
            querystring.parse(((runtime as any)._WEAPPS_HISTORY.location.search || '').split('?')[1] || '') || {};
        } catch (e) {
          queryObj = { ...$page.dataset?.params };
        }

        Object.keys(params)
          .map((keyPath) => {
            const matched = keyPath.match(/(^.*?)[.[]/);
            if (matched) {
              return matched[0];
            }
            return keyPath;
          })
          .forEach((key) => {
            queryObj[key] = $page.dataset?.params?.[key];
          });

        app.redirectTo({
          pageId: $page.id,
          packageName: $page.__internal__.packageName,
          params: queryObj,
        });
      },
      _dataBinds: {},
      invokeComponentMethod({ component, method, params }) {
        let userWidget: UserWidget;
        if (typeof component === 'string') {
          const widgetData = lodashGet($page.widgets, component);
          if (Array.isArray(widgetData)) {
            throw new Error(`调用方法失败：id为${component}的组件拥有多个实例`);
          }
          userWidget = widgetData?._userWidget;
        } else {
          userWidget = component;
        }
        const action = userWidget?.custom?.[method];
        if (typeof action !== 'function') {
          throw new Error(`调用方法失败：未找到id为${userWidget?.sys.id}下的方法${method}`);
        }
        return action(params);
      },
    },
    {
      get(obj, prop) {
        return prop === '$page' ? obj : obj[prop];
      },
    },
  );
  return $page;
}

// 分app 和 wx 挂载app
function mountAPIs(sdks) {
  Object.keys(sdks).forEach((item) => {
    let action = sdks[item];
    if (!(item in ACTIONS_KEY) && !(item in ROUTER_KEY)) {
      app[item] = action;
    }
  });
  return app;
}

export interface ICompositeComponentAPI extends IBaseState {
  __internal__?: {
    $w?: any;
    defaultCssUnit?: 'rpx' | 'rem' | 'px';
  };
  id: string;
  handler?: {
    [key: string]: any;
  };
  props: {
    [key: string]: any;
  };
  widgets?: {
    [key: string]: any;
  };
  _dataBinds?: {
    [key: string]: any;
  };
  _instanceRef: { current: any };
  compConfig?: any;
}

export function createCompApi({
  id,
  ref,
  state = observable({}),
  computed = {},
  _dataBinds = {},
  compConfig = {},
  node,
  events,
  props,
  handler = {},
  propsData,
  eventFlows,
}): ICompositeComponentAPI {
  const { $w, app } = (getWedaAPI() as any) || {};

  const $comp = new Proxy(
    {
      id,
      __internal__: {
        defaultCssUnit: undefined,
        $w: undefined,
        eventFlows: undefined,
      },
      _dataBinds,
      compConfig,
      widgets: observable({}),
      node,
      // lib: { const: constObj, tools: toolsObj },
      get props() {
        const { forwardRef, ...restProps } = props || {};
        return { ...restProps, events, data: propsData };
      },
      dataset: undefined,
      get state() {
        return state;
      },
      computed: {},
      get _instanceRef() {
        return ref;
      },
      get methods() {
        return ref.current?.methods;
      },
      set methods(value) {
        if (!ref.current) {
          ref.current = {};
        }
        ref.current.methods = value;
      },
    },
    {
      get: (obj, prop) => {
        return prop === '$WEAPPS_COMP' || prop === '$comp' ? obj : obj[prop];
      },
    },
  );

  const context = {
    $w: new Proxy($w || {}, {
      get(_, prop) {
        /**
         * 代理可以访问到当前 page
         */
        if (prop === '$page' || prop === 'page') {
          return app?.__internal__.activePage;
        }

        // 代理页面级 query
        if ($comp.dataset?.query?.[prop]) {
          return $comp.dataset.query[prop];
        }

        // 代理页面 flow
        if ($comp.__internal__.eventFlows?.[prop]) {
          return $comp.__internal__.eventFlows[prop];
        }

        // 尝试代理页面级别组件实例
        let pageWidget = $comp.widgets?.[prop];
        if (pageWidget) {
          return pageWidget._userWidget;
        }

        return app?.__internal__?.$w?.[prop];
      },
    }),
    $app: app,
    $page: new Proxy(
      {},
      {
        get(target, p) {
          return getWedaAPI()?.app.utils.getCurrentPage()?.[p];
        },
      },
    ) as IPageAPI,
  };

  $comp.__internal__.$w = context.$w;
  $comp.__internal__.eventFlows = generateEventFlows(eventFlows, context, {
    isComposite: true,
    looseError: true,
    useScopeWidgetData: true,
  });

  return Object.assign($comp, {
    computed: createComputed(computed, $comp),
    handler: bindFuncObj(handler || {}, $comp),
  });
}
